@@ -1,95 +1,225 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "qmljsmodelmanager.h"
 #include "qmljstoolsconstants.h"
 #include "qmljsplugindumper.h"
 #include "qmljsfindexportedcpptypes.h"
+#include "qmljssemanticinfo.h"
+#include "qmljsbundleprovider.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 #include <coreplugin/mimedatabase.h>
 #include <coreplugin/messagemanager.h>
-#include <cplusplus/ModelManagerInterface.h>
+#include <cpptools/ModelManagerInterface.h>
 #include <cplusplus/CppDocument.h>
 #include <qmljs/qmljscontext.h>
 #include <qmljs/qmljsbind.h>
+#include <qmljs/qmljsbundle.h>
 #include <qmljs/parser/qmldirparser_p.h>
 #include <texteditor/itexteditor.h>
 #include <texteditor/basetexteditor.h>
+#include <projectexplorer/buildconfiguration.h>
+#include <projectexplorer/kit.h>
+#include <projectexplorer/kitinformation.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/session.h>
+#include <projectexplorer/target.h>
+#include <projectexplorer/toolchain.h>
 #include <qtsupport/baseqtversion.h>
+#include <qtsupport/qtkitinformation.h>
+#include <qtsupport/qmldumptool.h>
+#include <qtsupport/qtsupportconstants.h>
+#include <utils/hostosinfo.h>
+#include <extensionsystem/pluginmanager.h>
 
-#include <QtCore/QDir>
-#include <QtCore/QFile>
-#include <QtCore/QFileInfo>
-#include <QtCore/QLibraryInfo>
-#include <QtCore/QtConcurrentRun>
-#include <qtconcurrent/runextensions.h>
-#include <QtCore/QTextStream>
-#include <QtCore/QCoreApplication>
-#include <QtCore/QTimer>
+#include <QDir>
+#include <QFile>
+#include <QFileInfo>
+#include <QLibraryInfo>
+#include <QtConcurrentRun>
+#include <utils/runextensions.h>
+#include <QTextStream>
+#include <QCoreApplication>
+#include <QTimer>
+#include <QRegExp>
 
-#include <QtCore/QDebug>
+#include <QDebug>
 
 using namespace QmlJS;
 using namespace QmlJSTools;
 using namespace QmlJSTools::Internal;
 
+
+ModelManagerInterface::ProjectInfo QmlJSTools::defaultProjectInfoForProject(
+        ProjectExplorer::Project *project)
+{
+    ModelManagerInterface::ProjectInfo projectInfo(project);
+    ProjectExplorer::Target *activeTarget = 0;
+    if (project) {
+        Core::MimeDatabase *db = Core::ICore::mimeDatabase();
+        QList<Core::MimeGlobPattern> globs;
+        QList<Core::MimeType> mimeTypes = db->mimeTypes();
+        foreach (const Core::MimeType &mimeType, mimeTypes)
+            if (mimeType.type() == QLatin1String(Constants::QML_MIMETYPE)
+                    || mimeType.subClassesOf().contains(QLatin1String(Constants::QML_MIMETYPE)))
+                globs << mimeType.globPatterns();
+        if (globs.isEmpty())
+            globs << Core::MimeGlobPattern(QRegExp(QLatin1String(".*\\.(?:qbs|qml|qmltypes|qmlproject)$")));
+        foreach (const QString &filePath
+                 , project->files(ProjectExplorer::Project::ExcludeGeneratedFiles))
+            foreach (const Core::MimeGlobPattern &glob, globs)
+                if (glob.regExp().exactMatch(filePath))
+                    projectInfo.sourceFiles << filePath;
+        activeTarget = project->activeTarget();
+    }
+    ProjectExplorer::Kit *activeKit = activeTarget ? activeTarget->kit() :
+                                           ProjectExplorer::KitManager::instance()->defaultKit();
+    QtSupport::BaseQtVersion *qtVersion = QtSupport::QtKitInformation::qtVersion(activeKit);
+
+    bool preferDebugDump = false;
+    bool setPreferDump = false;
+    projectInfo.tryQmlDump = false;
+
+    if (activeTarget) {
+        if (ProjectExplorer::BuildConfiguration *bc = activeTarget->activeBuildConfiguration()) {
+            preferDebugDump = bc->buildType() == ProjectExplorer::BuildConfiguration::Debug;
+            setPreferDump = true;
+        }
+    }
+    if (!setPreferDump && qtVersion)
+        preferDebugDump = (qtVersion->defaultBuildConfig() & QtSupport::BaseQtVersion::DebugBuild);
+    if (qtVersion && qtVersion->isValid()) {
+        projectInfo.tryQmlDump = project && (
+                    qtVersion->type() == QLatin1String(QtSupport::Constants::DESKTOPQT)
+                    || qtVersion->type() == QLatin1String(QtSupport::Constants::SIMULATORQT));
+        projectInfo.qtQmlPath = qtVersion->qmakeProperty("QT_INSTALL_QML");
+        projectInfo.qtImportsPath = qtVersion->qmakeProperty("QT_INSTALL_IMPORTS");
+        projectInfo.qtVersionString = qtVersion->qtVersionString();
+    }
+
+    if (projectInfo.tryQmlDump) {
+        ProjectExplorer::ToolChain *toolChain =
+                ProjectExplorer::ToolChainKitInformation::toolChain(activeKit);
+        QtSupport::QmlDumpTool::pathAndEnvironment(project, qtVersion,
+                                                   toolChain,
+                                                   preferDebugDump, &projectInfo.qmlDumpPath,
+                                                   &projectInfo.qmlDumpEnvironment);
+        projectInfo.qmlDumpHasRelocatableFlag = qtVersion->hasQmlDumpWithRelocatableFlag();
+    } else {
+        projectInfo.qmlDumpPath.clear();
+        projectInfo.qmlDumpEnvironment.clear();
+        projectInfo.qmlDumpHasRelocatableFlag = true;
+    }
+    setupProjectInfoQmlBundles(projectInfo);
+    return projectInfo;
+}
+
+void QmlJSTools::setupProjectInfoQmlBundles(ModelManagerInterface::ProjectInfo &projectInfo)
+{
+    ProjectExplorer::Target *activeTarget = 0;
+    if (projectInfo.project) {
+        activeTarget = projectInfo.project->activeTarget();
+    }
+    ProjectExplorer::Kit *activeKit = activeTarget
+            ? activeTarget->kit() : ProjectExplorer::KitManager::instance()->defaultKit();
+    QHash<QString, QString> replacements;
+    replacements.insert(QLatin1String("$(QT_INSTALL_IMPORTS)"), projectInfo.qtImportsPath);
+    replacements.insert(QLatin1String("$(QT_INSTALL_QML)"), projectInfo.qtQmlPath);
+
+    QList<IBundleProvider *> bundleProviders =
+            ExtensionSystem::PluginManager::getObjects<IBundleProvider>();
+
+    foreach (IBundleProvider *bp, bundleProviders) {
+        if (bp)
+            bp->mergeBundlesForKit(activeKit, projectInfo.activeBundle, replacements);
+    }
+    projectInfo.extendedBundle = projectInfo.activeBundle;
+
+    if (projectInfo.project) {
+        QSet<ProjectExplorer::Kit *> currentKits;
+        foreach (const ProjectExplorer::Target *t, projectInfo.project->targets())
+            if (t->kit())
+                currentKits.insert(t->kit());
+        currentKits.remove(activeKit);
+        foreach (ProjectExplorer::Kit *kit, currentKits) {
+            foreach (IBundleProvider *bp, bundleProviders)
+                if (bp)
+                    bp->mergeBundlesForKit(kit, projectInfo.extendedBundle, replacements);
+        }
+    }
+}
+
 static QStringList environmentImportPaths();
 
+static void mergeSuffixes(QStringList &l1, const QStringList &l2)
+{
+    if (!l2.isEmpty())
+        l1 = l2;
+}
+
 QmlJS::Document::Language QmlJSTools::languageOfFile(const QString &fileName)
 {
-    QStringList jsSuffixes("js");
-    QStringList qmlSuffixes("qml");
+    QStringList jsSuffixes(QLatin1String("js"));
+    QStringList qmlSuffixes(QLatin1String("qml"));
+    QStringList qmlProjectSuffixes(QLatin1String("qmlproject"));
+    QStringList jsonSuffixes(QLatin1String("json"));
+    QStringList qbsSuffixes(QLatin1String("qbs"));
 
     if (Core::ICore::instance()) {
-        Core::MimeDatabase *db = Core::ICore::instance()->mimeDatabase();
-        Core::MimeType jsSourceTy = db->findByType(Constants::JS_MIMETYPE);
-        jsSuffixes = jsSourceTy.suffixes();
-        Core::MimeType qmlSourceTy = db->findByType(Constants::QML_MIMETYPE);
-        qmlSuffixes = qmlSourceTy.suffixes();
+        Core::MimeDatabase *db = Core::ICore::mimeDatabase();
+        Core::MimeType jsSourceTy = db->findByType(QLatin1String(Constants::JS_MIMETYPE));
+        mergeSuffixes(jsSuffixes, jsSourceTy.suffixes());
+        Core::MimeType qmlSourceTy = db->findByType(QLatin1String(Constants::QML_MIMETYPE));
+        mergeSuffixes(qmlSuffixes, qmlSourceTy.suffixes());
+        Core::MimeType qbsSourceTy = db->findByType(QLatin1String(Constants::QBS_MIMETYPE));
+        mergeSuffixes(qbsSuffixes, qbsSourceTy.suffixes());
+        Core::MimeType qmlProjectSourceTy = db->findByType(QLatin1String(Constants::QMLPROJECT_MIMETYPE));
+        mergeSuffixes(qbsSuffixes, qmlProjectSourceTy.suffixes());
+        Core::MimeType jsonSourceTy = db->findByType(QLatin1String(Constants::JSON_MIMETYPE));
+        mergeSuffixes(jsonSuffixes, jsonSourceTy.suffixes());
     }
 
     const QFileInfo info(fileName);
     const QString fileSuffix = info.suffix();
     if (jsSuffixes.contains(fileSuffix))
         return QmlJS::Document::JavaScriptLanguage;
-    if (qmlSuffixes.contains(fileSuffix))
+    if (qbsSuffixes.contains(fileSuffix))
+        return QmlJS::Document::QmlQbsLanguage;
+    if (qmlSuffixes.contains(fileSuffix) || qmlProjectSuffixes.contains(fileSuffix))
         return QmlJS::Document::QmlLanguage;
+    if (jsonSuffixes.contains(fileSuffix))
+        return QmlJS::Document::JsonLanguage;
     return QmlJS::Document::UnknownLanguage;
 }
 
@@ -97,9 +227,9 @@ QStringList QmlJSTools::qmlAndJsGlobPatterns()
 {
     QStringList pattern;
     if (Core::ICore::instance()) {
-        Core::MimeDatabase *db = Core::ICore::instance()->mimeDatabase();
-        Core::MimeType jsSourceTy = db->findByType(Constants::JS_MIMETYPE);
-        Core::MimeType qmlSourceTy = db->findByType(Constants::QML_MIMETYPE);
+        Core::MimeDatabase *db = Core::ICore::mimeDatabase();
+        Core::MimeType jsSourceTy = db->findByType(QLatin1String(Constants::JS_MIMETYPE));
+        Core::MimeType qmlSourceTy = db->findByType(QLatin1String(Constants::QML_MIMETYPE));
 
         QStringList pattern;
         foreach (const Core::MimeGlobPattern &glob, jsSourceTy.globPatterns())
@@ -107,14 +237,13 @@ QStringList QmlJSTools::qmlAndJsGlobPatterns()
         foreach (const Core::MimeGlobPattern &glob, qmlSourceTy.globPatterns())
             pattern << glob.regExp().pattern();
     } else {
-        pattern << "*.qml" << "*.js";
+        pattern << QLatin1String("*.qml") << QLatin1String("*.js");
     }
     return pattern;
 }
 
 ModelManager::ModelManager(QObject *parent):
         ModelManagerInterface(parent),
-        m_core(Core::ICore::instance()),
         m_pluginDumper(new PluginDumper(this))
 {
     m_synchronizer.setCancelOnWait(true);
@@ -126,6 +255,7 @@ ModelManager::ModelManager(QObject *parent):
 
     qRegisterMetaType<QmlJS::Document::Ptr>("QmlJS::Document::Ptr");
     qRegisterMetaType<QmlJS::LibraryInfo>("QmlJS::LibraryInfo");
+    qRegisterMetaType<QmlJSTools::SemanticInfo>("QmlJSTools::SemanticInfo");
 
     loadQmlTypeDescriptions();
 
@@ -151,17 +281,15 @@ void ModelManager::delayedInitialization()
     }
 
     ProjectExplorer::SessionManager *sessionManager = ProjectExplorer::ProjectExplorerPlugin::instance()->session();
-    if (sessionManager) {
-        connect(sessionManager, SIGNAL(projectRemoved(ProjectExplorer::Project*)),
-                this, SLOT(removeProjectInfo(ProjectExplorer::Project*)));
-    }
+    connect(sessionManager, SIGNAL(projectRemoved(ProjectExplorer::Project*)),
+            this, SLOT(removeProjectInfo(ProjectExplorer::Project*)));
 }
 
 void ModelManager::loadQmlTypeDescriptions()
 {
     if (Core::ICore::instance()) {
-        loadQmlTypeDescriptions(Core::ICore::instance()->resourcePath());
-        loadQmlTypeDescriptions(Core::ICore::instance()->userResourcePath());
+        loadQmlTypeDescriptions(Core::ICore::resourcePath());
+        loadQmlTypeDescriptions(Core::ICore::userResourcePath());
     }
 }
 
@@ -203,19 +331,16 @@ void ModelManager::loadQmlTypeDescriptions(const QString &resourcePath)
 ModelManagerInterface::WorkingCopy ModelManager::workingCopy() const
 {
     WorkingCopy workingCopy;
-    if (!m_core)
+    if (!Core::ICore::instance())
         return workingCopy;
 
-    Core::EditorManager *editorManager = m_core->editorManager();
-
-    foreach (Core::IEditor *editor, editorManager->openedEditors()) {
-        const QString key = editor->file()->fileName();
+    foreach (Core::IEditor *editor, Core::ICore::editorManager()->openedEditors()) {
+        const QString key = editor->document()->fileName();
 
         if (TextEditor::ITextEditor *textEditor = qobject_cast<TextEditor::ITextEditor*>(editor)) {
             if (textEditor->context().contains(ProjectExplorer::Constants::LANG_QMLJS)) {
-                if (TextEditor::BaseTextEditorWidget *ed = qobject_cast<TextEditor::BaseTextEditorWidget *>(textEditor->widget())) {
+                if (TextEditor::BaseTextEditorWidget *ed = qobject_cast<TextEditor::BaseTextEditorWidget *>(textEditor->widget()))
                     workingCopy.insert(key, ed->toPlainText(), ed->document()->revision());
-                }
             }
         }
     }
@@ -244,9 +369,8 @@ void ModelManager::updateSourceFiles(const QStringList &files,
 QFuture<void> ModelManager::refreshSourceFiles(const QStringList &sourceFiles,
                                                bool emitDocumentOnDiskChanged)
 {
-    if (sourceFiles.isEmpty()) {
+    if (sourceFiles.isEmpty())
         return QFuture<void>();
-    }
 
     QFuture<void> result = QtConcurrent::run(&ModelManager::parse,
                                               workingCopy(), sourceFiles,
@@ -267,8 +391,8 @@ QFuture<void> ModelManager::refreshSourceFiles(const QStringList &sourceFiles,
     m_synchronizer.addFuture(result);
 
     if (sourceFiles.count() > 1) {
-        m_core->progressManager()->addTask(result, tr("Indexing"),
-                        Constants::TASK_INDEX);
+        Core::ICore::progressManager()->addTask(result, tr("Indexing"),
+                        QLatin1String(Constants::TASK_INDEX));
     }
 
     return result;
@@ -349,8 +473,9 @@ void ModelManager::updateProjectInfo(const ProjectInfo &pinfo)
     }
     updateSourceFiles(newFiles, false);
 
-    // dump builtin types if the shipped definitions are probably outdated
-    if (QtSupport::QtVersionNumber(pinfo.qtVersionString) > QtSupport::QtVersionNumber(4, 7, 3))
+    // dump builtin types if the shipped definitions are probably outdated and the
+    // Qt version ships qmlplugindump
+    if (QtSupport::QtVersionNumber(pinfo.qtVersionString) > QtSupport::QtVersionNumber(4, 8, 5))
         m_pluginDumper->loadBuiltinTypes(pinfo);
 
     emit projectInfoUpdated(pinfo);
@@ -360,6 +485,7 @@ void ModelManager::updateProjectInfo(const ProjectInfo &pinfo)
 void ModelManager::removeProjectInfo(ProjectExplorer::Project *project)
 {
     ProjectInfo info(project);
+    info.sourceFiles.clear();
     // update with an empty project info to clear data
     updateProjectInfo(info);
 
@@ -369,6 +495,16 @@ void ModelManager::removeProjectInfo(ProjectExplorer::Project *project)
     }
 }
 
+ModelManagerInterface::ProjectInfo ModelManager::projectInfoForPath(QString path)
+{
+    QMutexLocker locker(&m_mutex);
+
+    foreach (const ProjectInfo &p, m_projects)
+        if (p.sourceFiles.contains(path))
+            return p;
+    return ProjectInfo();
+}
+
 void ModelManager::emitDocumentChangedOnDisk(Document::Ptr doc)
 { emit documentChangedOnDisk(doc); }
 
@@ -464,17 +600,16 @@ static bool findNewQmlLibraryInPath(const QString &path,
         return false;
     }
 
-#ifdef Q_OS_WIN
-    // QTCREATORBUG-3402 - be case sensitive even here?
-#endif
+    if (Utils::HostOsInfo::isWindowsHost()) {
+        // QTCREATORBUG-3402 - be case sensitive even here?
+    }
 
     // found a new library!
     qmldirFile.open(QFile::ReadOnly);
     QString qmldirData = QString::fromUtf8(qmldirFile.readAll());
 
     QmlDirParser qmldirParser;
-    qmldirParser.setSource(qmldirData);
-    qmldirParser.parse();
+    qmldirParser.parse(qmldirData);
 
     const QString libraryPath = QFileInfo(qmldirFile).absolutePath();
     newLibraries->insert(libraryPath);
@@ -504,7 +639,7 @@ static void findNewQmlLibrary(
     QSet<QString> *scannedPaths,
     QSet<QString> *newLibraries)
 {
-    QString libraryPath = QString("%1.%2.%3").arg(
+    QString libraryPath = QString::fromLatin1("%1.%2.%3").arg(
                 path,
                 QString::number(version.majorVersion()),
                 QString::number(version.minorVersion()));
@@ -512,7 +647,7 @@ static void findNewQmlLibrary(
                 libraryPath, snapshot, modelManager,
                 importedFiles, scannedPaths, newLibraries);
 
-    libraryPath = QString("%1.%2").arg(
+    libraryPath = QString::fromLatin1("%1.%2").arg(
                 path,
                 QString::number(version.majorVersion()));
     findNewQmlLibraryInPath(
@@ -593,7 +728,7 @@ void ModelManager::parse(QFutureInterface<void> &future,
             }
         }
 
-        Document::Ptr doc = Document::create(fileName, language);
+        Document::MutablePtr doc = Document::create(fileName, language);
         doc->setEditorRevision(documentRevision);
         doc->setSource(contents);
         doc->parse();
@@ -625,7 +760,7 @@ void ModelManager::parse(QFutureInterface<void> &future,
 // Check whether fileMimeType is the same or extends knownMimeType
 bool ModelManager::matchesMimeType(const Core::MimeType &fileMimeType, const Core::MimeType &knownMimeType)
 {
-    Core::MimeDatabase *db = Core::ICore::instance()->mimeDatabase();
+    Core::MimeDatabase *db = Core::ICore::mimeDatabase();
 
     const QStringList knownTypeNames = QStringList(knownMimeType.type()) + knownMimeType.aliases();
 
@@ -644,21 +779,30 @@ bool ModelManager::matchesMimeType(const Core::MimeType &fileMimeType, const Cor
 
 QStringList ModelManager::importPaths() const
 {
+    QMutexLocker l(&m_mutex);
     return m_allImportPaths;
 }
 
+QmlLanguageBundles ModelManager::activeBundles() const
+{
+    QMutexLocker l(&m_mutex);
+    return m_activeBundles;
+}
+
+QmlLanguageBundles ModelManager::extendedBundles() const
+{
+    QMutexLocker l(&m_mutex);
+    return m_extendedBundles;
+}
+
 static QStringList environmentImportPaths()
 {
     QStringList paths;
 
     QByteArray envImportPath = qgetenv("QML_IMPORT_PATH");
 
-#if defined(Q_OS_WIN)
-    QLatin1Char pathSep(';');
-#else
-    QLatin1Char pathSep(':');
-#endif
-    foreach (const QString &path, QString::fromLatin1(envImportPath).split(pathSep, QString::SkipEmptyParts)) {
+    foreach (const QString &path, QString::fromLatin1(envImportPath)
+             .split(Utils::HostOsInfo::pathListSeparator(), QString::SkipEmptyParts)) {
         QString canonicalPath = QDir(path).canonicalPath();
         if (!canonicalPath.isEmpty() && !paths.contains(canonicalPath))
             paths.append(canonicalPath);
@@ -669,18 +813,54 @@ static QStringList environmentImportPaths()
 
 void ModelManager::updateImportPaths()
 {
-    m_allImportPaths.clear();
+    QStringList allImportPaths;
+    QmlLanguageBundles activeBundles;
+    QmlLanguageBundles extendedBundles;
     QMapIterator<ProjectExplorer::Project *, ProjectInfo> it(m_projects);
     while (it.hasNext()) {
         it.next();
         foreach (const QString &path, it.value().importPaths) {
             const QString canonicalPath = QFileInfo(path).canonicalFilePath();
             if (!canonicalPath.isEmpty())
-                m_allImportPaths += canonicalPath;
+                allImportPaths += canonicalPath;
         }
     }
-    m_allImportPaths += m_defaultImportPaths;
-    m_allImportPaths.removeDuplicates();
+    it.toFront();
+    while (it.hasNext()) {
+        it.next();
+        activeBundles.mergeLanguageBundles(it.value().activeBundle);
+        foreach (Document::Language l, it.value().activeBundle.languages()) {
+            foreach (const QString &path, it.value().activeBundle.bundleForLanguage(l)
+                 .searchPaths().stringList()) {
+                const QString canonicalPath = QFileInfo(path).canonicalFilePath();
+                if (!canonicalPath.isEmpty())
+                    allImportPaths += canonicalPath;
+            }
+        }
+    }
+    it.toFront();
+    while (it.hasNext()) {
+        it.next();
+        extendedBundles.mergeLanguageBundles(it.value().extendedBundle);
+        foreach (Document::Language l, it.value().extendedBundle.languages()) {
+            foreach (const QString &path, it.value().extendedBundle.bundleForLanguage(l)
+                     .searchPaths().stringList()) {
+                const QString canonicalPath = QFileInfo(path).canonicalFilePath();
+                if (!canonicalPath.isEmpty())
+                    allImportPaths += canonicalPath;
+            }
+        }
+    }
+    allImportPaths += m_defaultImportPaths;
+    allImportPaths.removeDuplicates();
+
+    {
+        QMutexLocker l(&m_mutex);
+        m_allImportPaths = allImportPaths;
+        m_activeBundles = activeBundles;
+        m_extendedBundles = extendedBundles;
+    }
+
 
     // check if any file in the snapshot imports something new in the new paths
     Snapshot snapshot = _validSnapshot;
@@ -704,7 +884,7 @@ void ModelManager::maybeQueueCppQmlTypeUpdate(const CPlusPlus::Document::Ptr &do
 {
     // avoid scanning documents without source code available
     doc->keepSourceAndAST();
-    if (doc->source().isEmpty()) {
+    if (doc->utf8Source().isEmpty()) {
         doc->releaseSourceAndAST();
         return;
     }
@@ -797,8 +977,6 @@ ModelManager::CppDataHash ModelManager::cppData() const
 LibraryInfo ModelManager::builtins(const Document::Ptr &doc) const
 {
     ProjectExplorer::SessionManager *sessionManager = ProjectExplorer::ProjectExplorerPlugin::instance()->session();
-    if (!sessionManager)
-        return LibraryInfo();
     ProjectExplorer::Project *project = sessionManager->projectForFile(doc->fileName());
     if (!project)
         return LibraryInfo();
